

var QuickPasswords_TabURIregexp = {
	get _thunderbirdRegExp() {
		delete this._thunderbirdRegExp;
		return this._thunderbirdRegExp = new RegExp("^http://quickpasswords.mozdev.org/");
	}
};

// open the new content tab for displaying support info, see
// https://developer.mozilla.org/en/Thunderbird/Content_Tabs
var QuickPasswords_TabURIopener = {

	openURLInTab: function (URL) {
		try {
			var sTabMode="";
			var tabmail;
			tabmail = document.getElementById("tabmail");
			if (!tabmail) {
				// Try opening new tabs in an existing 3pane window
				var mail3PaneWindow = Components.classes["@mozilla.org/appshell/window-mediator;1"]
										 .getService(Components.interfaces.nsIWindowMediator)
										 .getMostRecentWindow("mail:3pane");
				if (mail3PaneWindow) {
					tabmail = mail3PaneWindow.document.getElementById("tabmail");
					mail3PaneWindow.focus();
				}
			}
			if (tabmail) {
				sTabMode = (QuickPasswords.Util.Application() == "Thunderbird" && QuickPasswords.Util.Appver()>=3) ? "contentTab" : "3pane";
				tabmail.openTab(sTabMode,
				{contentPage: URL, clickHandler: "specialTabs.siteClickHandler(event, QuickPasswords_TabURIregexp._thunderbirdRegExp);"});
			}
			else
				window.openDialog("chrome://messenger/content/", "_blank",
								  "chrome,dialog=no,all", null,
			  { tabType: "contentTab", tabParams: {contentPage: URL, clickHandler: "specialTabs.siteClickHandler(event, QuickPasswords_TabURIregexp._thunderbirdRegExp);", id:"QuickFolders_Weblink"} } );
		}
		catch(e) { return false; }
		return true;
	}
};

QuickPasswords.Util = {
	QuickPasswords_CURRENTVERSION : '1.5',
	ConsoleService: null,
    mAppver: null,
    mAppName: null,
    mHost: null,
    lastTime: 0,
    myVersion: null,

    $: function(id) {
        return document.getElementById(id);
    } ,

    getAddon: function(aId) {
	    var em = Components.classes["@mozilla.org/extensions/manager;1"]
                  .getService(Components.interfaces.nsIExtensionManager);
        return em.getItemForID(aId);
    } ,

    Version: function() {
	    return this.myVersion
	    	   ?
	    	   this.myVersion
	    	   :
	    	   this.QuickPasswords_CURRENTVERSION;
    } ,

	// dedicated function for email clients which don't support tabs
	// and for secured pages (donation page).
	openLinkInBrowserForced: function(linkURI) {
		try {
			this.logDebug("openLinkInBrowserForced (" + linkURI + ")");
			if (QuickPasswords.Util.Application()=='SeaMonkey') {
				var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
				browser = windowManager.getMostRecentWindow( "navigator:browser" );
				if (browser) {
					let URI = linkURI;
					setTimeout(function() {  browser.currentTab = browser.getBrowser().addTab(URI); browser.currentTab.reload(); }, 250);
				}
				else
					window.openDialog(getBrowserURL(), "_blank", "all,dialog=no", linkURI, null, 'QuickPasswords update');

				return;
			}
			var service = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
				.getService(Components.interfaces.nsIExternalProtocolService);
			var ioservice = Components.classes["@mozilla.org/network/io-service;1"].
						getService(Components.interfaces.nsIIOService);
			var uri = ioservice.newURI(linkURI, null, null);
			service.loadURI(uri);
		}
		catch(e) { this.logDebug("openLinkInBrowserForced (" + linkURI + ") " + e.toString()); }
	},


	// moved from options.js
	// use this to follow a href that did not trigger the browser to open (from a XUL file)
	openLinkInBrowser: function(evt,linkURI) {
		if (QuickPasswords.Util.Appver()>=3 && QuickPasswords.Util.Application()=='Thunderbird') {
			var service = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
				.getService(Components.interfaces.nsIExternalProtocolService);
			var ioservice = Components.classes["@mozilla.org/network/io-service;1"].
						getService(Components.interfaces.nsIIOService);
			service.loadURI(ioservice.newURI(linkURI, null, null));
			if(null!=evt)
				evt.stopPropagation();
		}
		else
			this.openLinkInBrowserForced(linkURI);
	},

	// moved from options.js (then called
	openURL: function(evt,URL) { // workaround for a bug in TB3 that causes href's not be followed anymore.
		var ioservice,iuri,eps;

		if (QuickPasswords.Util.Appver()<3 && QuickPasswords.Util.Application()=='Thunderbird'
			|| QuickPasswords.Util.Application()=='SeaMonkey'
			|| QuickPasswords.Util.Application()=='Postbox')
		{
			this.openLinkInBrowserForced(URL);
			if(null!=evt) evt.stopPropagation();
		}
		else {
			if (QuickPasswords_TabURIopener.openURLInTab(URL) && null!=evt) {
				evt.preventDefault();
				evt.stopPropagation();
			}
		}
	},



	checkFirstRun: function() {
		QuickPasswords.Util.logDebug("checkFirstRun");
		var prev = -1, firstrun = true;
		var showFirsts = true, debugFirstRun = false;

		var svc = Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService);
		var ssPrefs = svc.getBranch("extensions.quickpasswords.");

		var current = QuickPasswords.Util.Version();
		QuickPasswords.Util.logDebug("Current QuickPasswords Version: " + current);
		try {
			QuickPasswords.Util.logDebugOptional ("firstRun","try to get setting: getCharPref(version)");
			try { prev = ssPrefs.getCharPref("version"); } catch (e) { prev = "?"; } ;

			QuickPasswords.Util.logDebugOptional ("firstRun","try to get setting: getBoolPref(firstrun)");
			try { firstrun = ssPrefs.getBoolPref("firstRun"); } catch (e) { firstrun = true; }

			// enablefirstruns=false - allows start pages to be turned off for partners
			QuickPasswords.Util.logDebugOptional ("firstRun","try to get setting: getBoolPref(enablefirstruns)");
			try { showFirsts = ssPrefs.getBoolPref("enablefirstruns"); } catch (e) { showFirsts = true; }


			QuickPasswords.Util.logDebugOptional ("firstRun", "Settings retrieved:"
					+ "\nprevious version=" + prev
					+ "\ncurrent version=" + current
					+ "\nfirstrun=" + firstrun
					+ "\nshowfirstruns=" + showFirsts
					+ "\ndebugFirstRun=" + debugFirstRun);

		}
		catch(e) {

			alert("QuickPasswords exception in quickpasswords_util.js: " + e.message
				+ "\n\ncurrent: " + current
				+ "\nprev: " + prev
				+ "\nfirstrun: " + firstrun
				+ "\nshowFirstRuns: " + showFirsts
				+ "\ndebugFirstRun: " + debugFirstRun);

		}
		finally {
			QuickPasswords.Util.logDebugOptional ("firstRun","finally - firstrun=" + firstrun);

			// AG if this is a pre-release, cut off everything from "pre" on... e.g. 1.9pre11 => 1.9
			var pre=0;
			var pureVersion=current;
			if (0<(pre=current.indexOf('pre'))) {
				pureVersion = current.substring(0,pre);
			}
			QuickPasswords.Util.logDebugOptional ("firstRun","finally - pureVersion=" + pureVersion);
			// change this depending on the branch
			var versionPage = "http://quickpasswords.mozdev.org/version.html#" + pureVersion;
			QuickPasswords.Util.logDebugOptional ("firstRun","finally - versionPage=" + versionPage);

			if (firstrun){
				QuickPasswords.Util.logDebugOptional ("firstRun","set firstRun=false and store version " + current);
				ssPrefs.setBoolPref("firstRun",false);
				ssPrefs.setCharPref("version",current); // store current (simplified) version!

				if (showFirsts) {
					// Insert code for first run here
					// on very first run, we go to the index page - welcome blablabla
					QuickPasswords.Util.logDebugOptional ("firstRun","setTimeout for content tab (index.html)");
					window.setTimeout(function() {
						QuickPasswords.Util.openURL(null, "http://quickpasswords.mozdev.org/index.html");
					}, 1500); //Firefox 2 fix - or else tab will get closed (leave it in....)

				}

			}
			else { // this section does not get loaded if its a fresh install.
				if (prev!=current) { // VERSION UPDATE!
					QuickPasswords.Util.logDebugOptional ("firstRun","prev!=current -> upgrade case.");
					// upgrade case!!
					ssPrefs.setCharPref("version",current);

					if (showFirsts) {
						// version is different => upgrade (or conceivably downgrade)
						QuickPasswords.Util.logDebugOptional ("firstRun","open tab for version history + browser for donation" + current);
						window.setTimeout(function(){
							// display version history
							QuickPasswords.Util.openURL(null, versionPage);

							// display donation page (can be disabled; I will send out method to all donators and anyone who asks me for it)
							if ((QuickPasswords.Preferences.getBoolPrefQF("donateNoMore")))
								QuickPasswords.Util.logDebugOptional ("firstRun","Jump to donations page disabled by user");
							else
								QuickPasswords.Util.openURL("http://quickpasswords.mozdev.org/donate.html"); // show donation page!
						}, 1500); //Firefox 2 fix - or else tab will get closed
					}
				}
				else
					QuickPasswords.Util.logDebugOptional ("firstRun","prev!=current -> just a reload of same version - prev=" + prev + ", current = " + current);


			}

		}

	} ,


	checkVersionFirstRun: function() {
		var aId = "QuickPasswords@axelg.com";
		if(!Components.classes["@mozilla.org/extensions/manager;1"])
		{
			Components.utils.import("resource://gre/modules/AddonManager.jsm");
			setTimeout (function () {
					AddonManager.getAddonByID(aId,
						function(addon) {
							// This is an asynchronous callback function that might not be called immediately, ah well...
							QuickPasswords.Util.myVersion = addon.version;
							QuickPasswords.Util.logDebug("AddonManager retrieved Version number: " + addon.version);
							QuickPasswords.Util.checkFirstRun();
						}
					);
			    }, 0);
	    }
	    else {
	    	QuickPasswords.Util.myVersion = this.getAddon(aId).version;
			QuickPasswords.Util.logDebug("Retrieved Version number from nsIExtensionManager (legacy): " + QuickPasswords.Util.myVersion);
			QuickPasswords.Util.checkFirstRun();
    	}
    },

    AppverFull: function() {
	    var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
	                    .getService(Components.interfaces.nsIXULAppInfo);
        return appInfo.version;
    },

    Appver: function() {
        if (null == this.mAppver) {
	    var appVer=this.AppverFull().substr(0,3); // only use 1st three letters - that's all we need for compatibility checking!
		    this.mAppver = parseFloat(appVer); // quick n dirty!
        }
		return this.mAppver;
    },

    Application: function() {
		if (null==this.mAppName) {
        var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
                        .getService(Components.interfaces.nsIXULAppInfo);
			const FIREFOX_ID = "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}";
			const THUNDERBIRD_ID = "{3550f703-e582-4d05-9a08-453d09bdfdc6}";
			const SEAMONKEY_ID = "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}";
			const POSTBOX_ID = "postbox@postbox-inc.com";
			switch(appInfo.ID) {
				case FIREFOX_ID:
				  return this.mAppName='Firefox';
				case THUNDERBIRD_ID:
				  return this.mAppName='Thunderbird';
				case SEAMONKEY_ID:
				  return this.mAppName='SeaMonkey';
				case POSTBOX_ID:
				  return this.mAppName='Postbox';
				default:
				  this.mAppName=appInfo.name;
				  this.logDebug ( 'Unknown Application: ' + appInfo.name);
				  return appInfo.name;
			}
        }
	    return this.mAppName;
    },

    HostSystem: function() {
		if (null==this.mHost) {
			var osString = Components.classes["@mozilla.org/xre/app-info;1"]
		                .getService(Components.interfaces.nsIXULRuntime).OS;
			this.mHost = osString.toLowerCase();
        }
        return this.mHost; // linux - winnt - darwin
    },

    copyStringToClipboard: function(sString) {
       var service = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
       var clipboardhelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
	   clipboardhelper.copyString(sString);
    },


    debugVar: function(value) {
        str = "Value: " + value + "\r\n";
        for(prop in value) {
            str += prop + " => " + value[prop] + "\r\n";
        }

        this.logDebug(str);
    },

    logTime: function() {
	    var timePassed = '';
	    try {
		    var end= new Date();
		    var endTime = end.getTime();
			var elapsed = new String(endTime  - this.lastTime); // time in milliseconds
			timePassed = '[' + elapsed + ' ms]   ';
		    this.lastTime = endTime; // remember last time
        }
        catch(e) {;}
	    return end.getHours() + ':' + end.getMinutes() + ':' + end.getSeconds() + '.' + end.getMilliseconds() + '  ' + timePassed;
    },

    logToConsole: function (msg) {
	  if (this.ConsoleService == null)
	    this.ConsoleService = Components.classes["@mozilla.org/consoleservice;1"]
	                               .getService(Components.interfaces.nsIConsoleService);
	  this.ConsoleService.logStringMessage("QuickPasswords " + this.logTime() + "\n"+ msg);
	},

    logDebug: function (msg) {
	  if (QuickPasswords.Preferences.isDebug())
	    this.logToConsole(msg);
	},

    logDebugOptional: function (option, msg) {
	  if (QuickPasswords.Preferences.isDebugOption(option))
	    this.logToConsole(msg);
	},

	about: function() {
		// show the built in about dialog:
		//window.opener.gExtensionsView.builder.rebuild();
		window.opener.gExtensionsViewController.doCommand('cmd_about');
	},

	onLoadOptions: function() {
		document.getElementById('qp-options-header-description').value=this.Version();
		// hide about button...
		if (!window.opener.gExtensionsViewController) {
			var about = document.getElementById("qp-About");
			if (about)
				about.style["visibility"]="hidden";
		}

	},

	displayOptions: function() {
		window.openDialog('chrome://quickpasswords/content/quickpassword_options.xul','quickfolders-passwords','chrome,titlebar,centerscreen,modal,resizable').focus();
	} ,

	showAboutConfig: function(filter) {

		const name = "Preferences:ConfigManager";
		const uri = "chrome://global/content/config.xul";

		var mediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
		var w = mediator.getMostRecentWindow(name);

		if (!w) {
			var watcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"].getService(Components.interfaces.nsIWindowWatcher);
			w = watcher.openWindow(null, uri, name, "chrome,resizable,centerscreen,width=500px,height=350px", null);
		}
		w.focus();
		w.setTimeout(
			function () {
				var flt = w.document.getElementById("textbox");
				if (flt) {
					 flt.value=filter;
					 flt.focus();
					 if (w.self.FilterPrefs)
					 w.self.FilterPrefs();
				}
			}, 300);
	},

	showVersionHistory: function(label, ask) {
		var pre=0;
		var current=label.value.toString();  // retrieve version number from label
		var pureVersion = current;
		if (0<(pre=current.indexOf('pre'))) {   // make sure to strip of any pre release labels
			pureVersion = current.substring(0,pre);
		}

		var sPrompt = QuickFolders.Util.getBundleString("qfConfirmVersionLink", "Display version history for QuickFolders")
		if (!ask || confirm(sPrompt + " " + pureVersion + "?")) {
			QuickFolders.Util.openURL(null, "http://quickpasswords.mozdev.org/version.html" + "#" + pureVersion);
		}
	}

}
